/*
 *  counter.js - A simple "odometer-style" counter.
 *
 *  Basé sur la version 0.5.1 de ffcounter.js:
 *  http://www.infocraft.com/projects/ffcounter/ffcounter.js
 *
 *  Copyright 2005 V.U.
 *  http://ipv6pourtous.free.fr/
 *  Copyright 2005 Infocraft
 *  http://www.infocraft.com/
 *
 *  Licence (la même que celle de ffcounter.js) :
 *  You may use this script in its entirety, modify it to suit your needs,
 *  adapt its functions, or just use it for inspiration.  However, some form
 *  of acknowledgement, especially a link back to http://www.infocraft.com/, is
 *  always appreciated.
 */

/***************************  Adjustable Parameters ***************************/

var counter_url = "http://ipv6pourtous.free.fr/rss/compteur/";	// The URL of your counter mirror.
var initial_rate = 0.1;		// The initial rate, in downloads/second.

var counter_id = "counter";	// The id of the counter element.
var signrate_id = "signrate";	// The id of the sign rate display element.
var last_count_id = "lastcount";	// The id of the last count display element.
var last_time_id = "lasttime";	// The id of the last time display element.
var status_id = "counterstatus";	// The id of the counter status element.
var timer_id = "countertimer";	// The id of the counter timer element.
var tick_id = "ticktime";	// The id of the tick rate element.

/***************************     Static Globals     ***************************/

var selected_option = 3;
var get_count_timer = 0;
var set_status_timer = 0;
var update_counter_timer = 0;

var current_count = 0.0;

var last_count = 0.0;
var last_time = null;

var sign_rate = 0.0;
var tick_time = 0;
var request = null;
var counter_on = false;

/***************************    Public Functions    ***************************/

// This should be called from your HTML, probably as an onload="get_count();"
// event in your <body> tag.
function get_count()
{
	var delay = 60000;

	load_xml(counter_url);
	get_count_timer = setTimeout("get_count()", delay);

	if (document.getElementById(timer_id)) {
		set_status(Math.round(delay / 1000));
	}
}

function restart_counter(new_option)
{
	if (selected_option == new_option) {
		return;
	}

	// Cancel what's already running
	if (request) {
		request.abort();
		request = null;
		set_element(status_id, "Aborted");
	}

	if (get_count_timer != 0) {
		clearTimeout(get_count_timer);
		get_count_timer = 0;
	}

	if (set_status_timer != 0) {
		clearTimeout(set_status_timer);
		set_status_timer = 0;
	}

	if (update_counter_timer != 0) {
		clearTimeout(update_counter_timer);
		update_counter_timer = 0;
	}

	// Reinitialize
	selected_option = new_option;

	set_element(counter_id, "Chargement...");
	set_element(signrate_id, "Chargement...");
	set_element(last_count_id, "Chargement...");
	set_element(last_time_id, "Chargement...");
	set_element(status_id, "Chargement...");
	set_element(timer_id, "Chargement...");
	set_element(tick_id, "Chargement...");

	current_count = 0.0;
	last_count = 0.0;
	last_time = null;
	sign_rate = 0.0;
	tick_time = 0;
	counter_on = false;

	// Restart
	get_count();
}

/***************************    Private Functions   ***************************/

// Advances the status counter, indicating the length until the next increment.
function set_status(time)
{
	var timer = document.getElementById(timer_id);

	timer.firstChild.data = time;

	time--;
	if (time > 0) {
		set_status_timer = setTimeout("set_status(" + time + ")", 1000);
	}
}


// Adapted from the good people at ADC:
// http://developer.apple.com/internet/webcontent/xmlhttpreq.html
// this starts loading the XML data and sets up the process_request handler.
function load_xml(url) 
{
	set_element(status_id, "Sending Request");

	// branch for native XMLHttpRequest object
	if (window.XMLHttpRequest) {
		request = new XMLHttpRequest();
		request.onreadystatechange = process_request;
		request.open("GET", url, true);
		request.send(null);

	// branch for IE/Windows ActiveX version
	} else if (window.ActiveXObject) {
		request = new ActiveXObject("Microsoft.XMLHTTP");
		if (request) {
			request.onreadystatechange = process_request;
			request.open("GET", url, true);
			request.send();
		}
	}
}

// This function is called once the request state changes.  It pulls the
// relevant data from the response and sends to update_count().
function process_request()
{
	var response;
	var time;
	var count_item;
	var count;

	set_element(status_id, "Request status: " + request.readyState);

	if (request.readyState == 4) {
		if (request.status == 200) {
			response = request.responseXML;
			time = response.getElementsByTagName('pubDate')[0].firstChild.data;
			count_item = response.getElementsByTagName('item')[selected_option];
			count = count_item.getElementsByTagName('description')[0].firstChild.data;
			update_count(time, count);
		} else {
			alert("Error: " + request.statusText());
		}

		set_element(status_id, "Received: " + (request.statusText ? request.statusText : request.status));
		request = null;
	}
}

// Now that the data from the XML request has been extracted, this updates the
// global variables with that data.
function update_count(time, count)
{
	time = new Date(time);
	count = parseFloat(count.replace(/\D/g,''));

	if (initial_rate > 0 || last_time) {

		if (last_time && time - last_time != 0) {
			sign_rate = calculate_rate(time, count);
		} else {
			sign_rate = initial_rate;
		}

		if (sign_rate != 0) {
			tick_time = Math.round(1000.0 / sign_rate);
		} else {
			tick_time = 0;
		}

		set_element(tick_id, tick_time);
		set_element(signrate_id, Math.round(sign_rate * 10) / 10.0
				+ (last_time ? "" : " (est.)"));
	}

	last_time = time;
	last_count = count;
	current_count = count;

	set_element(last_count_id, format_count(last_count));
	set_element(last_time_id, format_time(last_time));

	if (!counter_on) {
		counter_on = true;
		update_counter();
	}
}

function update_counter()
{
	
	set_element(counter_id, format_count(Math.round(current_count)));
	current_count++;

	if (tick_time != 0) {
		update_counter_timer = setTimeout("update_counter()", tick_time);
	} else {
		counter_on = false;
	}
}

/***************************    Utility Functions   ***************************/

// Calculates the number of downloads per update interval.
function calculate_rate(new_time, new_count)
{
	var time_delta = (new_time - last_time) / 1000.0;
	var count_delta = new_count - last_count;
	var rate = count_delta / time_delta;

	return rate;
}

// Formats a count for display by making it into a string and inserting spaces.
function format_count(count)
{
	count = count.toString();

	for (var i = count.length - 3; i > 0 ; i -= 3) {
		count = count.slice(0, i) + ' ' + count.slice(i, count.length);
	}

	return count;
}

// Formats a time for display.
function format_time(time)
{
	var hour = time.getHours();
	var minutes = time.getMinutes();
	var seconds = time.getSeconds();

	if (minutes < 10)
		minutes = "0" + minutes ;
	if (seconds < 10)
		seconds = "0" + seconds;

	return hour + ":" + minutes + ":" + seconds;
}

// Sets an HTML element with element_id to the given content.
function set_element(element_id, content)
{
	var element;

	if (element = document.getElementById(element_id))
		element.firstChild.data = content;
}
